#include <arpa/inet.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <fcntl.h> 
#include "arm-probe.h"

#define FIELDS_PER_CHANNEL 8

int configure(struct aep *aep, const char *dev_filepath,
	const char *config_filepath, struct aep_channel *wch)
{
	char buf[256];
	char marshall[10];
	int len = 0;
	int pos = 0;
	int more = 1;
	int fd = open(config_filepath, O_RDONLY);
	char c;
	enum actions {
		APCA_GET_NAME,
		APCA_CHECK_DEVPATH,
		APCA_MATCH_DEVPATH,
		APCA_PULL_CHANNELS,
	};
	enum actions actions = APCA_GET_NAME;
	enum parser {
		ACPP_FIRST,
		ACPP_SKIP,
		ACPP_EAT,
		ACPP_SWALLOW_WHITE
	};
	enum parser parser = ACPP_FIRST;
	enum parser entry_parser;
	int index = 0;
	int copy_pos = 0;
	int field;
	struct aep_channel *ch;
	int wfd = -1;
	char temp_config[512];
	char linebuf[512];
	int no_copy = 0;

	if (wch) {
		strncpy(temp_config, config_filepath, sizeof temp_config - 2);
		temp_config[sizeof temp_config - 2] = '\0';
		temp_config[strlen(temp_config) + 1] = '\0';
		temp_config[strlen(temp_config)] = '~';
		unlink(temp_config);
		wfd = open(temp_config, O_RDWR | O_CREAT, 0664);
		if (wfd < 0) {
			fprintf(stderr, "Unable to open temp config file %s\n", temp_config);
			return -1;
		}
	}

	if (aep)
		ch = &aep->ch[0];

	if (fd < 0) {
		fprintf(stderr, "unable to open config file %s\n",
				config_filepath);
		return -1;
	}

	while (more) {
		if (pos == len) {
			len = read(fd, buf, sizeof buf);
			if (len <= 0) {
				more = 0;
				continue;
			}
			pos = 0;
		}

		c = buf[pos++];
		entry_parser = parser;

		if (!no_copy && wfd > 0)
			write(wfd, &c, 1);

		switch (parser) {
		case ACPP_FIRST:
			if (c == '#') {
				parser = ACPP_SKIP;
				continue;
			}
			if (c != '\n')
				parser = ACPP_EAT;
			break;
		case ACPP_SKIP:
			if (c == '\n')
				parser = ACPP_FIRST;
			continue;
		case ACPP_EAT:
			if (c == '\n')
				parser = ACPP_FIRST;
			break;
		case ACPP_SWALLOW_WHITE:
			if (c == '\n') {
				parser = ACPP_FIRST;
				continue;
			}
			if (c != ' ' && c != '\t')
				parser = ACPP_EAT;
			else
				continue;
			break;
		}


		switch (actions) {
		case APCA_GET_NAME:
			if (parser == ACPP_FIRST) {
				configuration_name[copy_pos] = '\0';
				copy_pos = 0;
				actions = APCA_CHECK_DEVPATH;
				if (aep == NULL)
					return 0;
			} else
				if (copy_pos < (sizeof configuration_name) - 1)
					configuration_name[copy_pos++] = c;
			break;
		case APCA_CHECK_DEVPATH:
			if (entry_parser == ACPP_FIRST) {
				if (c != '/') {
					if (c != '\n')
						parser = ACPP_SKIP;
					continue;
				}
				copy_pos = 1;
				actions = APCA_MATCH_DEVPATH;
			}
			break;

		case APCA_MATCH_DEVPATH:
			if (parser == ACPP_FIRST) {
				if (dev_filepath[copy_pos] != '\0') {
					parser = ACPP_SKIP;
					actions = APCA_CHECK_DEVPATH;
					copy_pos = 0;
					break;
				}
				actions = APCA_PULL_CHANNELS;
				copy_pos = 0;
				no_copy = ch == wch;
				break;
			}
			if (dev_filepath[copy_pos++] != c) {
				parser = ACPP_SKIP;
				actions = APCA_CHECK_DEVPATH;
				copy_pos = 0;
				break;
			}
			break;

		case APCA_PULL_CHANNELS:
			if (entry_parser == ACPP_FIRST) {
				if (c != '\t' && c !=' ') {
					more = 0;
					continue;
				}
				parser = ACPP_SWALLOW_WHITE;
				copy_pos = 0;
				field = 0;
				break;
			}

			if (parser == ACPP_FIRST) {
				index++;
				if (ch == wch) {
					sprintf(linebuf, " %s\t%f\t%f\t%f\t%f\t%f\t%d\t%s\n",
						ch->channel_name,
						ch->rshunt,
						ch->voffset[0],
						ch->vnoise[0],
						ch->voffset[1],
						ch->vnoise[1],
						ch->pretrigger_ms,
						ch->channel_name_pretty);
					fprintf(stderr, "Updating config with \"%s\"\n", linebuf);
					write(wfd, linebuf, strlen(linebuf));
				}

				ch++;
				if (index == CHANNELS_PER_PROBE) {
					more = 0;
					continue;
				}
				no_copy = ch == wch;
			}

			if (c == '\t' || c == ' ') {
				field++;
				parser = ACPP_SWALLOW_WHITE;
				copy_pos = 0;
				break;
			}

			if (no_copy)
				break;

			switch (field) {
			case 0:
				if (copy_pos >= sizeof ch->channel_name - 1)
					break;
				ch->channel_name[copy_pos++] = c;
				ch->channel_name[copy_pos] = '\0';
				break;
			case 1:
			case 2:
			case 3:
			case 4:
			case 5:
			case 6:
				if (copy_pos >= sizeof marshall - 1)
					break;
				marshall[copy_pos++] = c;
				marshall[copy_pos] = '\0';
				switch (field) {
				case 1:
					ch->rshunt = atof(marshall);
					break;
				case 2:
					ch->voffset[0] = atof(marshall);
					break;
				case 3:
					ch->vnoise[0] = atof(marshall);
					break;
				case 4:
					ch->voffset[1] = atof(marshall);
					break;
				case 5:
					ch->vnoise[1] = atof(marshall);
					break;
				case 6:
					ch->pretrigger_ms = atoi(marshall);
					ch->ring_samples =
						(ch->pretrigger_ms + ms_pretrigger) * 10;
					ch->pretrig_ring = NULL;
					ch->head = 0;
					ch->tail = 0;
					if (ch->ring_samples)
						ch->pretrig_ring = malloc (
						     sizeof(struct samples) *
							     ch->ring_samples);						
					break;
				default:
					break;
				}	
				break;
			case 7:
				if (copy_pos >= sizeof ch->channel_name_pretty - 1)
					break;
				ch->channel_name_pretty[copy_pos++] = c;
				ch->channel_name_pretty[copy_pos] = '\0';
				break;
			default:
				fprintf(stderr, "Unable to parse config\n");
				close(fd);
				return -1;
			}
			break;
		}
	}

	if (wfd > 0) {
		more = len > 0;
		while (more) {
			if (pos == len) {
	                        len = read(fd, buf, sizeof buf);
	                        if (len <= 0) {                                      
	                                more = 0;
	                                continue;                                       
				}                                                       
				pos = 0;
			}

			c = buf[pos++];
			write(wfd, &c, 1);
		}

		close(wfd);
	}

	close(fd);

	if (wfd > 0) {
		unlink(config_filepath);
		rename(temp_config, config_filepath);
	}

	return 0;
}


